#ifndef DASMTABLES_H
#define DASMTABLES_H

extern const char* const mnemonic_xx_cb[256];
extern const char* const mnemonic_cb[256];
extern const char* const mnemonic_ed[256];
extern const char* const mnemonic_xx[256];
extern const char* const mnemonic_main[256];

#endif // DASMTABLES_H
